import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.*;

/**
 * MapContainer es un contenedor de {@link MapPlane}, esta clase se encarga de 
 * generar la estructura de capas en base a {@link MapPlane}.
 * Adicionalmente puede almacenar otros MapContainer en una lista, para generar 
 * un arbol de MapContainer.
 * Para distinguir contenedores entre si, cada uno tiene asociado un ID unico 
 * correspondiente al numero de instancia generada.
 * @author Jaime
 */
public class MapContainer {

    /**
     * Numero de planos contenidos en un objeto MapContainer 
     */
   public static final int MAP_PLANE_NUMBERS = 3;
   private static int MapContainerInstanceCounter = 0;
   private final int ContainerID;
   private ArrayList<MapContainer> MapContainerChildren = null;
   private final MapPlane[] MapPlanes = new MapPlane[MAP_PLANE_NUMBERS];
   private String MapName;
   private MapContainerView view;
   private final int width;
   private final int height;
   private Tileset MapTileset;
   private int TopLayer = 3;
   
    /**
     * Genera un nuevo MapContainer con las dimensiones y {@link Tileset} 
     * especificados. El nombre por defecto es "Map_<i>x</i>", con "<i>x</i>" 
     * correspondiente al numero de la instancia generada.
     * @param width Ancho en {@link Tileset.Tile} de cada {@link MapPlane} 
     * almacenado.
     * @param height Largo en {@link Tileset.Tile} de cada {@link MapPlane} 
     * almacenado.
     * @param MapTileset {@link Tileset} a utilizar por cada mapa.
     */
    public MapContainer(int width, int height, Tileset MapTileset){
      this.MapTileset = MapTileset;
      this.width = width;
      this.height = height;
      for(int i=0; i<MAP_PLANE_NUMBERS; i++){
         MapPlanes[i] = new MapPlane(width, height, MapTileset);
      }
      ContainerID = MapContainerInstanceCounter;
      MapContainerInstanceCounter++;
      MapName = "MAP_" + ContainerID;
      view = new MapContainerView();
   }
   
    /**
     * Genera un nuevo MapContainer con las dimensiones, nombre y 
     * {@link Tileset} especificados.
     * @param width Ancho en {@link Tileset.Tile} de cada {@link MapPlane} 
     * almacenado.
     * @param height Largo en {@link Tileset.Tile} de cada {@link MapPlane} 
     * almacenado.
     * @param MapTileset {@link Tileset} a utilizar por cada mapa.
     * @param name Nombre del MapContainer.
     */
    public MapContainer(int width, int height, Tileset MapTileset, String name){
      this(width, height, MapTileset);
      MapName = name;
   }
   
    /**
     * Entrega el ID del MapContainer
     * @return Entero con el ID del MapContainer
     */
    public int getContainerID(){
      return ContainerID; 
   }
   
    /**
     * Remueve el MapCointainer con el ID ContainerIndex.
     * @param ContainerIndex ID del MapContainer a remover.
     * @return true si el MapContainer fue removido con exito, false en caso 
     * contrario.
     */
    public boolean removeMapContainer(int ContainerIndex){
      boolean result = false;
      for(int i=0; i<MapContainerChildren.size(); i++){
         if(MapContainerChildren.get(i).getContainerID() == ContainerIndex){
            MapContainerChildren.remove(i);
            result = true;
         }
      }
      if(MapContainerChildren.isEmpty())
          MapContainerChildren = null;
      return result;
   }
   
    /**
     * Agrega un nuevo MapContainer a la lista interna de la instancia con los
     * parametros entregados.
     * @param width Ancho en {@link Tileset.Tile} del nuevo MapContainer.
     * @param height Largo en {@link Tileset.Tile} del nuevo MapContainer.
     * @param MapTileset {@link Tileset} del nuevo MapContainer.
     * @return Referencia al MapContainer creado.
     */
    public MapContainer addMapContainer(int width, int height, Tileset MapTileset){
      if(MapContainerChildren==null)
         MapContainerChildren = new ArrayList<MapContainer>();
      
      MapContainer newMap = new MapContainer(width, height, MapTileset);
      MapContainerChildren.add(newMap);
      return newMap;
   }
   
    /**
     * Agrega un nuevo MapContainer a la lista interna de la instancia con los
     * parametros entregados.
     * @param width Ancho en {@link Tileset.Tile} del nuevo MapContainer.
     * @param height Largo en {@link Tileset.Tile} del nuevo MapContainer.
     * @param MapTileset {@link Tileset} del nuevo MapContainer.
     * @param name Nombre del nuevo MapContainer.
     * @return Referencia al MapContainer creado.
     */
    public MapContainer addMapContainer(int width, int height, Tileset MapTileset, String name){
      if(MapContainerChildren==null)
         MapContainerChildren = new ArrayList<MapContainer>();   
   
      MapContainer newMap = new MapContainer(width, height, MapTileset, name);
      MapContainerChildren.add(newMap);
      return newMap;
   }   
   
    /**
     * Invoca a {@link #FloodLayer() MapPlane} en el plano superior.
     */
    public void FloodLayer(){
      MapPlanes[TopLayer].FloodMap();
   }
   
    /**
     * Retorna el nombre por defecto que tendra el siguiente MapContainer
     * @return String con el nombre del siguiente MapContainer.
     */
    public static String getNextContainerName(){
      return "MAP_" + MapContainerInstanceCounter;
   }
   
    /**
     * Retorna la vista de capas del MapContainer
     * @return JLayeredPane del MapContainer
     */
    public JLayeredPane getView(){
      setTopMapLayer(MAP_PLANE_NUMBERS);
      return view;
   }
   
    /**
     * Retorna la vista del {@link Tileset} de la instancia de MapContainer.
     * @return JScrollPane con la vista del {@link Tileset} utilizado.
     */
    public JScrollPane getTilesetView(){
      return MapTileset.getView();
   }
   
    /**
     * Coloca al {@link MapPlane} en la capa <i>layer</i> en el plano superior.
     * En caso de que layer este fuera de rango, no se ejecuta ninguna accion.
     * @param layer 
     */
    public void setTopMapLayer(int layer){
      if((layer<1) || (layer>MAP_PLANE_NUMBERS)){
         return;
      }
      view.rearrangeLayers(layer-1);
   }
   
   @Override
   public String toString(){
      return MapName;
   }
   
    /**
     * Retorna la lista de MapContainer almacenada.
     * @return ArrayList&lt;MapContainer&gt; con la lista de MapContainer
     */
    public ArrayList<MapContainer> getChildren(){
      return MapContainerChildren;
   }
   
    final private class MapContainerView extends JLayeredPane {
      MapContainerView(){
         rearrangeLayers(MAP_PLANE_NUMBERS-1);
         Dimension Final_size = new Dimension(Tileset.TILE_PIX_SIZE*width,Tileset.TILE_PIX_SIZE*height);
         this.setMaximumSize(Final_size);
         this.setPreferredSize(Final_size);
         this.addMouseListener(new TilePlacerListener());
      }
      
      private void rearrangeLayers(int TopLayer){
         MapContainer.this.TopLayer = TopLayer;
         SpringLayout layout = new SpringLayout();
         setLayout(layout);
         this.removeAll();
         
         ArrayList<MapPlane> ShuffledMapPlanes = new ArrayList<MapPlane>();//new MapPlane[MAP_PLANE_NUMBERS];
         
         for(int i=0; i<MAP_PLANE_NUMBERS; i++){
            if(i!=TopLayer)
               ShuffledMapPlanes.add(MapPlanes[i]);
         }
         
         ShuffledMapPlanes.add(MapPlanes[TopLayer]);
         
         for(int i=0; i<MAP_PLANE_NUMBERS; i++){
            add(ShuffledMapPlanes.get(i).getMapView(),new Integer(i));
            layout.putConstraint(SpringLayout.WEST, MapPlanes[i].getMapView(),
                     0,
                     SpringLayout.WEST, this);
            layout.putConstraint(SpringLayout.NORTH, MapPlanes[i].getMapView(),
                     0,
                     SpringLayout.NORTH, this);
         }
      }
      
      private class TilePlacerListener extends MouseAdapter{
         @Override public void mouseReleased(MouseEvent e) {
            int pos_x = e.getPoint().x/Tileset.TILE_PIX_SIZE;
            int pos_y = e.getPoint().y/Tileset.TILE_PIX_SIZE;
            
            MapPlane.MapView TopLayer = (MapPlane.MapView) getComponentsInLayer(MAP_PLANE_NUMBERS-1)[0];
            TopLayer.setNewMapTiles(pos_x,pos_y);
         }
      }
   }
}
